<?php

namespace Litecms\Statuses\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Statuses\Http\Requests\StatusesRequest;
use Litecms\Statuses\Interfaces\StatusesRepositoryInterface;
use Litecms\Statuses\Models\Statuses;

/**
 * Resource controller class for statuses.
 */
class StatusesResourceController extends BaseController
{

    /**
     * Initialize statuses resource controller.
     *
     * @param type StatusesRepositoryInterface $statuses
     *
     * @return null
     */
    public function __construct(StatusesRepositoryInterface $statuses)
    {
        parent::__construct();
        $this->repository = $statuses;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Statuses\Repositories\Criteria\StatusesResourceCriteria::class);
    }

    /**
     * Display a list of statuses.
     *
     * @return Response
     */
    public function index(StatusesRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Statuses\Repositories\Presenter\StatusesPresenter::class)
                ->$function();
        }

        $statuses = $this->repository->paginate();

        return $this->response->title(trans('statuses::statuses.names'))
            ->view('statuses::statuses.index', true)
            ->data(compact('statuses', 'view'))
            ->output();
    }

    /**
     * Display statuses.
     *
     * @param Request $request
     * @param Model   $statuses
     *
     * @return Response
     */
    public function show(StatusesRequest $request, Statuses $statuses)
    {

        if ($statuses->exists) {
            $view = 'statuses::statuses.show';
        } else {
            $view = 'statuses::statuses.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('statuses::statuses.name'))
            ->data(compact('statuses'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new statuses.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(StatusesRequest $request)
    {

        $statuses = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('statuses::statuses.name')) 
            ->view('statuses::statuses.create', true) 
            ->data(compact('statuses'))
            ->output();
    }

    /**
     * Create new statuses.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(StatusesRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $statuses                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('statuses::statuses.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('statuses/statuses/' . $statuses->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/statuses/statuses'))
                ->redirect();
        }

    }

    /**
     * Show statuses for editing.
     *
     * @param Request $request
     * @param Model   $statuses
     *
     * @return Response
     */
    public function edit(StatusesRequest $request, Statuses $statuses)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('statuses::statuses.name'))
            ->view('statuses::statuses.edit', true)
            ->data(compact('statuses'))
            ->output();
    }

    /**
     * Update the statuses.
     *
     * @param Request $request
     * @param Model   $statuses
     *
     * @return Response
     */
    public function update(StatusesRequest $request, Statuses $statuses)
    {
        try {
            $attributes = $request->all();

            $statuses->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('statuses::statuses.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('statuses/statuses/' . $statuses->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('statuses/statuses/' . $statuses->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the statuses.
     *
     * @param Model   $statuses
     *
     * @return Response
     */
    public function destroy(StatusesRequest $request, Statuses $statuses)
    {
        try {

            $statuses->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('statuses::statuses.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('statuses/statuses/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('statuses/statuses/' . $statuses->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple statuses.
     *
     * @param Model   $statuses
     *
     * @return Response
     */
    public function delete(StatusesRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('statuses::statuses.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('statuses/statuses'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/statuses/statuses'))
                ->redirect();
        }

    }

    /**
     * Restore deleted statuses.
     *
     * @param Model   $statuses
     *
     * @return Response
     */
    public function restore(StatusesRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('statuses::statuses.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/statuses/statuses'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/statuses/statuses/'))
                ->redirect();
        }

    }

}
